@@ -33,6 +33,7 @@ struct _GsFlatpak {
 	AsAppScope		 scope;
 	GsPlugin		*plugin;
 	XbSilo			*silo;
+	GRWLock			 silo_lock;
 	gchar			*id;
 	guint			 changed_id;
 };
@@ -162,6 +163,7 @@ gs_flatpak_set_update_permissions (GsFlatpak *self, GsApp *app, FlatpakInstalled
 	g_autoptr(GBytes) bytes = NULL;
 	g_autoptr(GKeyFile) keyfile = NULL;
 	GsAppPermissions permissions;
+	g_autoptr(GError) error_local = NULL;
 
 	old_bytes = flatpak_installed_ref_load_metadata (FLATPAK_INSTALLED_REF (xref), NULL, NULL);
 	old_keyfile = g_key_file_new ();
@@ -174,14 +176,21 @@ gs_flatpak_set_update_permissions (GsFlatpak *self, GsApp *app, FlatpakInstalled
 	                                                         gs_app_get_origin (app),
 	                                                         FLATPAK_REF (xref),
 	                                                         NULL,
-	                                                         NULL);
-	keyfile = g_key_file_new ();
-	g_key_file_load_from_data (keyfile,
-	                           g_bytes_get_data (bytes, NULL),
-	                           g_bytes_get_size (bytes),
-	                           0, NULL);
+	                                                         &error_local);
+	if (bytes == NULL) {
+		g_debug ("Failed to get metadata for remote ‘%s’: %s",
+			 gs_app_get_origin (app), error_local->message);
+		g_clear_error (&error_local);
+		permissions = GS_APP_PERMISSIONS_UNKNOWN;
+	} else {
+		keyfile = g_key_file_new ();
+		g_key_file_load_from_data (keyfile,
+			                   g_bytes_get_data (bytes, NULL),
+			                   g_bytes_get_size (bytes),
+			                   0, NULL);
 
-	permissions = perms_from_metadata (keyfile) & ~perms_from_metadata (old_keyfile);
+		permissions = perms_from_metadata (keyfile) & ~perms_from_metadata (old_keyfile);
+	}
 
 	/* no new permissions set */
 	if (permissions == GS_APP_PERMISSIONS_UNKNOWN)
@@ -499,8 +508,10 @@ gs_flatpak_add_apps_from_xremote (GsFlatpak *self,
 #else
 		g_autoptr(GError) error_local = NULL;
 		main_ref = gs_flatpak_get_xremote_main_ref (self, xremote, &error_local);
-		if (main_ref == NULL)
+		if (main_ref == NULL) {
 			g_warning ("failed to get main ref: %s", error_local->message);
+			g_clear_error (&error_local);
+		}
 #endif
 		if (main_ref != NULL) {
 			g_autoptr(XbBuilderFixup) fixup = NULL;
@@ -649,13 +660,18 @@ gs_flatpak_rescan_appstream_store (GsFlatpak *self,
 	g_autofree gchar *blobfn = NULL;
 	g_autoptr(GFile) file = NULL;
 	g_autoptr(GPtrArray) xremotes = NULL;
+	g_autoptr(GRWLockReaderLocker) reader_locker = NULL;
+	g_autoptr(GRWLockWriterLocker) writer_locker = NULL;
 	g_autoptr(XbBuilder) builder = xb_builder_new ();
 
+	reader_locker = g_rw_lock_reader_locker_new (&self->silo_lock);
 	/* everything is okay */
 	if (self->silo != NULL && xb_silo_is_valid (self->silo))
 		return TRUE;
+	g_clear_pointer (&reader_locker, g_rw_lock_reader_locker_free);
 
 	/* drat! silo needs regenerating */
+	writer_locker = g_rw_lock_writer_locker_new (&self->silo_lock);
 	g_clear_object (&self->silo);
 
 	/* verbose profiling */
@@ -800,7 +816,7 @@ gs_flatpak_refresh_appstream_remote (GsFlatpak *self,
 	g_autofree gchar *str = NULL;
 	g_autoptr(GsApp) app_dl = gs_app_new (gs_plugin_get_name (self->plugin));
 	g_autoptr(GsFlatpakProgressHelper) phelper = NULL;
-	g_autoptr(GError) local_error = NULL;
+	g_autoptr(GError) error_local = NULL;
 
 	/* TRANSLATORS: status text when downloading new metadata */
 	str = g_strdup_printf (_("Getting flatpak metadata for %s…"), remote_name);
@@ -810,11 +826,11 @@ gs_flatpak_refresh_appstream_remote (GsFlatpak *self,
 	if (!flatpak_installation_update_remote_sync (self->installation,
 						      remote_name,
 						      cancellable,
-						      &local_error)) {
+						      &error_local)) {
 		g_debug ("Failed to update metadata for remote %s: %s\n",
-			 remote_name, local_error->message);
-		gs_flatpak_error_convert (&local_error);
-		g_propagate_error (error, g_steal_pointer (&local_error));
+			 remote_name, error_local->message);
+		gs_flatpak_error_convert (&error_local);
+		g_propagate_error (error, g_steal_pointer (&error_local));
 		return FALSE;
 	}
 	phelper = gs_flatpak_progress_helper_new (self->plugin, app_dl);
@@ -1379,6 +1395,7 @@ gs_flatpak_add_updates (GsFlatpak *self, GsAppList *list,
 			g_debug ("Couldn't get the main app for updatable app extension %s: "
 				 "%s; adding the app itself to the updates list...",
 				 gs_app_get_unique_id (app), error_local->message);
+			g_clear_error (&error_local);
 			main_app = g_object_ref (app);
 		}
 		gs_app_set_state (main_app, AS_APP_STATE_UPDATABLE_LIVE);
@@ -1411,6 +1428,7 @@ gs_flatpak_add_updates (GsFlatpak *self, GsAppList *list,
 										  &error_local)) {
 					g_warning ("failed to get download size: %s",
 						   error_local->message);
+					g_clear_error (&error_local);
 					gs_app_set_size_download (main_app, GS_APP_SIZE_UNKNOWABLE);
 				} else {
 					gs_app_set_size_download (main_app, download_size);
@@ -1445,8 +1463,10 @@ gs_flatpak_refresh (GsFlatpak *self,
 	}
 
 	/* manually do this in case we created the first appstream file */
+	g_rw_lock_reader_lock (&self->silo_lock);
 	if (self->silo != NULL)
 		xb_silo_invalidate (self->silo);
+	g_rw_lock_reader_unlock (&self->silo_lock);
 
 	/* update AppStream metadata */
 	if (!gs_flatpak_refresh_appstream (self, cache_age, cancellable, error))
@@ -1636,20 +1656,17 @@ gs_flatpak_create_fake_ref (GsApp *app, GError **error)
 	return xref;
 }
 
-gboolean
-gs_flatpak_refine_app_state (GsFlatpak *self,
-			      GsApp *app,
-			      GCancellable *cancellable,
-			      GError **error)
+/* the _unlocked() version doesn't call gs_flatpak_rescan_appstream_store,
+ * in order to avoid taking the writer lock on self->silo_lock */
+static gboolean
+gs_flatpak_refine_app_state_unlocked (GsFlatpak *self,
+                                      GsApp *app,
+                                      GCancellable *cancellable,
+                                      GError **error)
 {
 	g_autoptr(FlatpakInstalledRef) ref = NULL;
-	g_autoptr(GError) error_local = NULL;
 	g_autoptr(GPtrArray) refs = NULL;
 
-	/* ensure valid */
-	if (!gs_flatpak_rescan_appstream_store (self, cancellable, error))
-		return FALSE;
-
 	/* already found */
 	if (gs_app_get_state (app) != AS_APP_STATE_UNKNOWN)
 		return TRUE;
@@ -1724,6 +1741,19 @@ gs_flatpak_refine_app_state (GsFlatpak *self,
 	return TRUE;
 }
 
+gboolean
+gs_flatpak_refine_app_state (GsFlatpak *self,
+                             GsApp *app,
+                             GCancellable *cancellable,
+                             GError **error)
+{
+	/* ensure valid */
+	if (!gs_flatpak_rescan_appstream_store (self, cancellable, error))
+		return FALSE;
+
+	return gs_flatpak_refine_app_state_unlocked (self, app, cancellable, error);
+}
+
 static GsApp *
 gs_flatpak_create_runtime (GsFlatpak *self, GsApp *parent, const gchar *runtime)
 {
@@ -1989,10 +2019,10 @@ gs_plugin_refine_item_size (GsFlatpak *self,
 
 		/* is the app_runtime already installed? */
 		app_runtime = gs_app_get_runtime (app);
-		if (!gs_flatpak_refine_app_state (self,
-						  app_runtime,
-						  cancellable,
-						  error))
+		if (!gs_flatpak_refine_app_state_unlocked (self,
+		                                           app_runtime,
+		                                           cancellable,
+		                                           error))
 			return FALSE;
 		if (gs_app_get_state (app_runtime) == AS_APP_STATE_INSTALLED) {
 			g_debug ("runtime %s is already installed, so not adding size",
@@ -2048,6 +2078,7 @@ gs_plugin_refine_item_size (GsFlatpak *self,
 		if (!ret) {
 			g_warning ("libflatpak failed to return application "
 				   "size: %s", error_local->message);
+			g_clear_error (&error_local);
 		}
 	}
 
@@ -2107,22 +2138,23 @@ gs_flatpak_refine_appstream (GsFlatpak *self,
 	return TRUE;
 }
 
-gboolean
-gs_flatpak_refine_app (GsFlatpak *self,
-		       GsApp *app,
-		       GsPluginRefineFlags flags,
-		       GCancellable *cancellable,
-		       GError **error)
+/* the _unlocked() version doesn't call gs_flatpak_rescan_appstream_store,
+ * in order to avoid taking the writer lock on self->silo_lock */
+static gboolean
+gs_flatpak_refine_app_unlocked (GsFlatpak *self,
+                                GsApp *app,
+                                GsPluginRefineFlags flags,
+                                GCancellable *cancellable,
+                                GError **error)
 {
 	AsAppState old_state = gs_app_get_state (app);
+	g_autoptr(GRWLockReaderLocker) locker = NULL;
 
 	/* not us */
 	if (gs_app_get_bundle_kind (app) != AS_BUNDLE_KIND_FLATPAK)
 		return TRUE;
 
-	/* ensure valid */
-	if (!gs_flatpak_rescan_appstream_store (self, cancellable, error))
-		return FALSE;
+	locker = g_rw_lock_reader_locker_new (&self->silo_lock);
 
 	/* always do AppStream properties */
 	if (!gs_flatpak_refine_appstream (self, app, self->silo, flags, error))
@@ -2135,7 +2167,7 @@ gs_flatpak_refine_app (GsFlatpak *self,
 	}
 
 	/* check the installed state */
-	if (!gs_flatpak_refine_app_state (self, app, cancellable, error)) {
+	if (!gs_flatpak_refine_app_state_unlocked (self, app, cancellable, error)) {
 		g_prefix_error (error, "failed to get state: ");
 		return FALSE;
 	}
@@ -2191,6 +2223,20 @@ gs_flatpak_refine_app (GsFlatpak *self,
 	return TRUE;
 }
 
+gboolean
+gs_flatpak_refine_app (GsFlatpak *self,
+		       GsApp *app,
+		       GsPluginRefineFlags flags,
+		       GCancellable *cancellable,
+		       GError **error)
+{
+	/* ensure valid */
+	if (!gs_flatpak_rescan_appstream_store (self, cancellable, error))
+		return FALSE;
+
+	return gs_flatpak_refine_app_unlocked (self, app, flags, cancellable, error);
+}
+
 gboolean
 gs_flatpak_refine_wildcard (GsFlatpak *self, GsApp *app,
 			    GsAppList *list, GsPluginRefineFlags refine_flags,
@@ -2200,6 +2246,7 @@ gs_flatpak_refine_wildcard (GsFlatpak *self, GsApp *app,
 	g_autofree gchar *xpath = NULL;
 	g_autoptr(GError) error_local = NULL;
 	g_autoptr(GPtrArray) components = NULL;
+	g_autoptr(GRWLockReaderLocker) locker = NULL;
 
 	/* not enough info to find */
 	id = gs_app_get_id (app);
@@ -2210,6 +2257,8 @@ gs_flatpak_refine_wildcard (GsFlatpak *self, GsApp *app,
 	if (!gs_flatpak_rescan_appstream_store (self, cancellable, error))
 		return FALSE;
 
+	locker = g_rw_lock_reader_locker_new (&self->silo_lock);
+
 	/* find all apps when matching any prefixes */
 	xpath = g_strdup_printf ("components/component/id[text()='%s']/..", id);
 	components = xb_silo_query (self->silo, xpath, 0, &error_local);
@@ -2229,7 +2278,7 @@ gs_flatpak_refine_wildcard (GsFlatpak *self, GsApp *app,
 		if (new == NULL)
 			return FALSE;
 		gs_flatpak_claim_app (self, new);
-		if (!gs_flatpak_refine_app (self, new, refine_flags, cancellable, error))
+		if (!gs_flatpak_refine_app_unlocked (self, new, refine_flags, cancellable, error))
 			return FALSE;
 		gs_app_subsume_metadata (new, app);
 		gs_app_list_add (list, new);
@@ -2614,6 +2663,7 @@ gs_flatpak_file_to_app_ref (GsFlatpak *self,
 		gs_plugin_event_set_error (event, error_local);
 		gs_plugin_event_add_flag (event, GS_PLUGIN_EVENT_FLAG_WARNING);
 		gs_plugin_report_event (self->plugin, event);
+		g_clear_error (&error_local);
 	}
 
 	/* get this now, as it's not going to be available at install time */
@@ -2666,13 +2716,19 @@ gs_flatpak_search (GsFlatpak *self,
 		   GError **error)
 {
 	g_autoptr(GsAppList) list_tmp = gs_app_list_new ();
+	g_autoptr(GRWLockReaderLocker) locker = NULL;
+
 	if (!gs_flatpak_rescan_appstream_store (self, cancellable, error))
 		return FALSE;
+
+	locker = g_rw_lock_reader_locker_new (&self->silo_lock);
 	if (!gs_appstream_search (self->plugin, self->silo, values, list_tmp,
 				  cancellable, error))
 		return FALSE;
+
 	gs_flatpak_claim_app_list (self, list_tmp);
 	gs_app_list_add_list (list, list_tmp);
+
 	return TRUE;
 }
 
@@ -2684,14 +2740,20 @@ gs_flatpak_add_category_apps (GsFlatpak *self,
 			      GError **error)
 {
 	g_autoptr(GsAppList) list_tmp = gs_app_list_new ();
+	g_autoptr(GRWLockReaderLocker) locker = NULL;
+
 	if (!gs_flatpak_rescan_appstream_store (self, cancellable, error))
 		return FALSE;
+
+	locker = g_rw_lock_reader_locker_new (&self->silo_lock);
 	if (!gs_appstream_add_category_apps (self->plugin, self->silo,
 					     category, list_tmp,
 					     cancellable, error))
 		return FALSE;
+
 	gs_flatpak_claim_app_list (self, list_tmp);
 	gs_app_list_add_list (list, list_tmp);
+
 	return TRUE;
 }
 
@@ -2701,8 +2763,12 @@ gs_flatpak_add_categories (GsFlatpak *self,
 			   GCancellable *cancellable,
 			   GError **error)
 {
+	g_autoptr(GRWLockReaderLocker) locker = NULL;
+
 	if (!gs_flatpak_rescan_appstream_store (self, cancellable, error))
 		return FALSE;
+
+	locker = g_rw_lock_reader_locker_new (&self->silo_lock);
 	return gs_appstream_add_categories (self->plugin, self->silo,
 					    list, cancellable, error);
 }
@@ -2714,12 +2780,18 @@ gs_flatpak_add_popular (GsFlatpak *self,
 			GError **error)
 {
 	g_autoptr(GsAppList) list_tmp = gs_app_list_new ();
+	g_autoptr(GRWLockReaderLocker) locker = NULL;
+
 	if (!gs_flatpak_rescan_appstream_store (self, cancellable, error))
 		return FALSE;
+
+	locker = g_rw_lock_reader_locker_new (&self->silo_lock);
 	if (!gs_appstream_add_popular (self->plugin, self->silo, list_tmp,
 				       cancellable, error))
 		return FALSE;
+
 	gs_app_list_add_list (list, list_tmp);
+
 	return TRUE;
 }
 
@@ -2730,12 +2802,18 @@ gs_flatpak_add_featured (GsFlatpak *self,
 			 GError **error)
 {
 	g_autoptr(GsAppList) list_tmp = gs_app_list_new ();
+	g_autoptr(GRWLockReaderLocker) locker = NULL;
+
 	if (!gs_flatpak_rescan_appstream_store (self, cancellable, error))
 		return FALSE;
+
+	locker = g_rw_lock_reader_locker_new (&self->silo_lock);
 	if (!gs_appstream_add_featured (self->plugin, self->silo, list_tmp,
 					cancellable, error))
 		return FALSE;
+
 	gs_app_list_add_list (list, list_tmp);
+
 	return TRUE;
 }
 
@@ -2747,12 +2825,18 @@ gs_flatpak_add_alternates (GsFlatpak *self,
 			   GError **error)
 {
 	g_autoptr(GsAppList) list_tmp = gs_app_list_new ();
+	g_autoptr(GRWLockReaderLocker) locker = NULL;
+
 	if (!gs_flatpak_rescan_appstream_store (self, cancellable, error))
 		return FALSE;
+
+	locker = g_rw_lock_reader_locker_new (&self->silo_lock);
 	if (!gs_appstream_add_alternates (self->plugin, self->silo, app, list_tmp,
 					  cancellable, error))
 		return FALSE;
+
 	gs_app_list_add_list (list, list_tmp);
+
 	return TRUE;
 }
 
@@ -2764,13 +2848,19 @@ gs_flatpak_add_recent (GsFlatpak *self,
 		       GError **error)
 {
 	g_autoptr(GsAppList) list_tmp = gs_app_list_new ();
+	g_autoptr(GRWLockReaderLocker) locker = NULL;
+
 	if (!gs_flatpak_rescan_appstream_store (self, cancellable, error))
 		return FALSE;
+
+	locker = g_rw_lock_reader_locker_new (&self->silo_lock);
 	if (!gs_appstream_add_recent (self->plugin, self->silo, list_tmp, age,
 				      cancellable, error))
 		return FALSE;
+
 	gs_flatpak_claim_app_list (self, list_tmp);
 	gs_app_list_add_list (list, list_tmp);
+
 	return TRUE;
 }
 
@@ -2823,6 +2913,7 @@ gs_flatpak_finalize (GObject *object)
 	g_object_unref (self->plugin);
 	g_hash_table_unref (self->broken_remotes);
 	g_mutex_clear (&self->broken_remotes_mutex);
+	g_rw_lock_clear (&self->silo_lock);
 
 	G_OBJECT_CLASS (gs_flatpak_parent_class)->finalize (object);
 }
@@ -2837,6 +2928,10 @@ gs_flatpak_class_init (GsFlatpakClass *klass)
 static void
 gs_flatpak_init (GsFlatpak *self)
 {
+	/* XbSilo needs external locking as we destroy the silo and build a new
+	 * one when something changes */
+	g_rw_lock_init (&self->silo_lock);
+
 	g_mutex_init (&self->broken_remotes_mutex);
 	self->broken_remotes = g_hash_table_new_full (g_str_hash, g_str_equal,
 						      g_free, NULL);
