@@ -3,25 +3,10 @@
  * Copyright (C) 2013 Richard Hughes <richard@hughsie.com>
  * Copyright (C) 2016 Kalev Lember <klember@redhat.com>
  *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * SPDX-License-Identifier: GPL-2.0+
  */
 
-#ifndef __GS_COMMON_H
-#define __GS_COMMON_H
+#pragma once
 
 #include <gio/gdesktopappinfo.h>
 #include <gtk/gtk.h>
@@ -60,7 +45,3 @@ gboolean	 gs_utils_list_has_app_fuzzy	(GsAppList	*list,
 						 GsApp		*app);
 
 G_END_DECLS
-
-#endif /* __GS_COMMON_H */
-
-/* vim: set noexpandtab: */
