@@ -28,9 +28,6 @@ void		 gs_screenshot_image_set_screenshot	(GsScreenshotImage	*ssimg,
 void		 gs_screenshot_image_set_size		(GsScreenshotImage	*ssimg,
 							 guint			 width,
 							 guint			 height);
-void		 gs_screenshot_image_set_use_desktop_background
-							(GsScreenshotImage	*ssimg,
-							 gboolean		 use_desktop_background);
 void		 gs_screenshot_image_load_async		(GsScreenshotImage	*ssimg,
 							 GCancellable		*cancellable);
 
